#include <proto.h>

#include "gtwindow.h"
#include "gterror.h"

/************************************************************************\
*
* GtWindow class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define GTWPG_EVENTMASK		(ButtonPressMask|ButtonReleaseMask|\
								PointerMotionMask|PointerMotionHintMask|Button1MotionMask|\
								Button2MotionMask|Button3MotionMask|Button4MotionMask|\
								Button5MotionMask|ButtonMotionMask|EnterWindowMask|\
								LeaveWindowMask)


/************************************************************************\
*
* Create and init pixmap.
*
\************************************************************************/
GtWindow *GtMkWindow(GtDisplay *d,GtTag *tgs)
{
	GtWindow *obj;
	
	if(!(obj=new GtWindow()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtWindow object!\n");
		return(NULL);
	}
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init pixmap.
*
\************************************************************************/
GtWindow *GtMkWindow(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtWindow *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkWindow(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Create empty window.
*
\************************************************************************/
GtWindow::GtWindow(void)
{
	parent=NULL;
	win=0;
	back=NULL;
	border=NULL;
	cmap=NULL;
	brpixel=0;
	bgpixel=0;
	rs=FALSE;
	nofreex=FALSE;
}

/************************************************************************\
*
* Destroy window.
*
\************************************************************************/
void GtWindow::Destroy(void)
{
	if(win)
		XDestroyWindow(dsp->XDsp(),win);
	GtDrawable::Destroy();
}

/************************************************************************\
*
* Setup window.
*
\************************************************************************/
sbool GtWindow::Init(GtDisplay *d,GtTag *tgs)
{
	GtTag *t;
	Window par;
	int xp=0,yp=0,dp=CopyFromParent;
	ulong xs=1,ys=1,bw=0,cl=CopyFromParent;
	Visual *vi=CopyFromParent;
	XSetWindowAttributes attr;
	ulong mask=0;
	
	if(!GtDrawable::Init(d,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Parent:
				parent=(GtWindow *)t->d.ptr;
				break;
			case GT_XPos:
				xp=t->d.val;
				break;
			case GT_YPos:
				yp=t->d.val;
				break;
			case GT_XSize:
				xs=t->d.val;
				break;
			case GT_YSize:
				ys=t->d.val;
				break;
			case GT_BWidth:
				bw=t->d.val;
				break;
			case GT_Depth:
				dp=t->d.val;
				break;
			case GT_Class:
				cl=t->d.val;
				break;
			case GT_Visual:
				vi=(Visual *)t->d.ptr;
				break;
			case GT_BackPixmap:
				if((back=(GtPixmap *)t->d.ptr))
					attr.background_pixmap=back->XPix();
				else
					attr.background_pixmap=None;
				mask|=CWBackPixmap;
				break;
			case GT_BackPixel:
				bgpixel=attr.background_pixel=t->d.val;
				mask|=CWBackPixel;
				break;
			case GT_BorderPixmap:
				if((border=(GtPixmap *)t->d.ptr))
					attr.border_pixmap=border->XPix();
				else
					attr.border_pixmap=None;
				mask|=CWBorderPixmap;
				break;
			case GT_BorderPixel:
				brpixel=attr.border_pixel=t->d.val;
				mask|=CWBackPixel;
				break;
			case GT_BitGravity:
				attr.bit_gravity=t->d.val;
				mask|=CWBitGravity;
				break;
			case GT_WinGravity:
				attr.win_gravity=t->d.val;
				mask|=CWWinGravity;
				break;
			case GT_BackingStore:
				attr.backing_store=t->d.val;
				mask|=CWBackingStore;
				break;
			case GT_BackingPlanes:
				attr.backing_planes=t->d.val;
				mask|=CWBackingPlanes;
				break;
			case GT_BackingPixel:
				attr.backing_pixel=t->d.val;
				mask|=CWBackingPixel;
				break;
			case GT_Override:
				attr.override_redirect=t->d.val;
				mask|=CWOverrideRedirect;
				break;
			case GT_SaveUnder:
				attr.save_under=t->d.val;
				mask|=CWSaveUnder;
				break;
			case GT_EventMask:
				attr.event_mask=t->d.val;
				mask|=CWEventMask;
				break;
			case GT_DontPropagate:
				attr.do_not_propagate_mask=t->d.val;
				mask|=CWDontPropagate;
				break;
			case GT_Colormap:
				if((cmap=(GtColormap *)t->d.ptr))
					attr.colormap=cmap->XCm();
				else
					attr.colormap=None;
				mask|=CWColormap;
				break;
			case GT_Cursor:
				if((cur=(GtCursor *)t->d.ptr))
					attr.cursor=cur->XCurs();
				else
					attr.cursor=None;
				mask|=CWCursor;
				break;
			case GT_Raised:
				if(t->d.val)
					rs=TRUE;
				else
					rs=FALSE;
				break;
			case GT_XWindow:
				win=(Window)t->d.ptr;
				nofreex=TRUE;
				break;
			case GT_DontFreeX:
				if(t->d.val)
					nofreex=TRUE;
				else
					nofreex=FALSE;
				break;
		}
	}
	if(win)
	{
		dr=win;
		return(TRUE);
	}
	if(!parent)
		par=dsp->XDefaultRootWindow();
	else
		par=parent->XWin();
	xs=(xs ? xs : 1);
	ys=(ys ? ys : 1);
	win=XCreateWindow(dsp->XDsp(),par,xp,yp,xs,ys,bw,dp,cl,vi,mask,&attr);
	if(!win)
	{
		GtError(this,GTEC_NOXRESOURCE,"No X-Window!\n");
		return(FALSE);
	}
	dr=win;
	return(TRUE);
}

/************************************************************************\
*
* Set window attributes.
*
\************************************************************************/
sbool GtWindow::Set(GtTag *tgs)
{
	GtTag *t;
	XSetWindowAttributes attr;
	XWindowChanges attr2;
	ulong mask=0,mask2=0;
	
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_XPos:
				attr2.x=t->d.val;
				mask2|=CWX;
				break;
			case GT_YPos:
				attr2.y=t->d.val;
				mask2|=CWY;
				break;
			case GT_XSize:
				if(t->d.val)
				{
					attr2.width=t->d.val;
					mask2|=CWWidth;
				}
				break;
			case GT_YSize:
				if(t->d.val)
				{
					attr2.height=t->d.val;
					mask2|=CWHeight;
				}
				break;
			case GT_BWidth:
				attr2.border_width=t->d.val;
				mask2|=CWBorderWidth;
				break;
			case GT_BackPixmap:
				if((back=(GtPixmap *)t->d.ptr))
					attr.background_pixmap=back->XPix();
				else
					attr.background_pixmap=None;
				mask|=CWBackPixmap;
				break;
			case GT_BackPixel:
				bgpixel=attr.background_pixel=t->d.val;
				mask|=CWBackPixel;
				break;
			case GT_BorderPixmap:
				if((border=(GtPixmap *)t->d.ptr))
					attr.border_pixmap=border->XPix();
				else
					attr.border_pixmap=None;
				mask|=CWBorderPixmap;
				break;
			case GT_BorderPixel:
				brpixel=attr.border_pixel=t->d.val;
				mask|=CWBorderPixel;
				break;
			case GT_BitGravity:
				attr.bit_gravity=t->d.val;
				mask|=CWBitGravity;
				break;
			case GT_WinGravity:
				attr.win_gravity=t->d.val;
				mask|=CWWinGravity;
				break;
			case GT_BackingStore:
				attr.backing_store=t->d.val;
				mask|=CWBackingStore;
				break;
			case GT_BackingPlanes:
				attr.backing_planes=t->d.val;
				mask|=CWBackingPlanes;
				break;
			case GT_BackingPixel:
				attr.backing_pixel=t->d.val;
				mask|=CWBackingPixel;
				break;
			case GT_Override:
				attr.override_redirect=t->d.val;
				mask|=CWOverrideRedirect;
				break;
			case GT_SaveUnder:
				attr.save_under=t->d.val;
				mask|=CWSaveUnder;
				break;
			case GT_EventMask:
				attr.event_mask=t->d.val;
				mask|=CWEventMask;
				break;
			case GT_DontPropagate:
				attr.do_not_propagate_mask=t->d.val;
				mask|=CWDontPropagate;
				break;
			case GT_Colormap:
				if((cmap=(GtColormap *)t->d.ptr))
					attr.colormap=cmap->XCm();
				else
					attr.colormap=None;
				mask|=CWColormap;
				break;
			case GT_Cursor:
				if((cur=(GtCursor *)t->d.ptr))
					attr.cursor=cur->XCurs();
				else
					attr.cursor=None;
				mask|=CWCursor;
				break;
			case GT_Raised:
				if(t->d.val)
					rs=TRUE;
				else
					rs=FALSE;
				break;
		}
	}
	if(mask)
		XChangeWindowAttributes(dsp->XDsp(),win,mask,&attr);
	if(mask2)
		XConfigureWindow(dsp->XDsp(),win,mask2,&attr2);
	return(TRUE);
}

/************************************************************************\
*
* Get window attributes.
*
\************************************************************************/
void GtWindow::Get(GtTag *tgs)
{
	GtTag *t;
	ulong *ptr;
	XWindowAttributes attr;
	
	GtDrawable::Get(tgs);
	XGetWindowAttributes(dsp->XDsp(),win,&attr);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(ulong *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Parent:
				*ptr=(ulong)parent;
				break;
			case GT_XPos:
				*ptr=attr.x;
				break;
			case GT_YPos:
				*ptr=attr.y;
				break;
			case GT_XSize:
				*ptr=attr.width;
				break;
			case GT_YSize:
				*ptr=attr.height;
				break;
			case GT_BWidth:
				*ptr=attr.border_width;
				break;
			case GT_Depth:
				*ptr=attr.depth;
				break;
			case GT_Class:
				*ptr=attr.c_class;
				break;
			case GT_Visual:
				*ptr=(ulong)attr.visual;
				break;
			case GT_BackPixmap:
				*ptr=(ulong)back;
				break;
			case GT_BackPixel:
				*ptr=bgpixel;
				break;
			case GT_BorderPixmap:
				*ptr=(ulong)border;
				break;
			case GT_BorderPixel:
				*ptr=brpixel;
				break;
			case GT_BitGravity:
				*ptr=attr.bit_gravity;
				break;
			case GT_WinGravity:
				*ptr=attr.win_gravity;
				break;
			case GT_BackingStore:
				*ptr=attr.backing_store;
				break;
			case GT_BackingPlanes:
				*ptr=attr.backing_planes;
				break;
			case GT_BackingPixel:
				*ptr=attr.backing_pixel;
				break;
			case GT_Override:
				*ptr=attr.override_redirect;
				break;
			case GT_SaveUnder:
				*ptr=attr.save_under;
				break;
			case GT_EventMask:
				*ptr=attr.your_event_mask;
				break;
			case GT_DontPropagate:
				*ptr=attr.do_not_propagate_mask;
				break;
			case GT_Colormap:
				*ptr=(ulong)cmap;
				break;
			case GT_Cursor:
				*ptr=(long)cur;
				break;
			case GT_XWindow:
				*ptr=(ulong)win;
				break;
			case GT_DontFreeX:
				*ptr=nofreex;
				break;
			case GT_Raised:
				*ptr=rs;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtWindow::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtWindow")==0)
		return(TRUE);
	else
		return(GtDrawable::IsSubClassOf(cn));
}

/************************************************************************\
*
* Map or unmap window.
*
\************************************************************************/
void GtWindow::Map(sbool map)
{
	if(map)
	{
		if(rs)
			XMapRaised(dsp->XDsp(),win);
		else
			XMapWindow(dsp->XDsp(),win);
	}
	else
		XUnmapWindow(dsp->XDsp(),win);
}

/************************************************************************\
*
* Grab pointer in window.
*
\************************************************************************/
sbool GtWindow::GrabPointer(Time tm,GtTag *tgs)
{
	GtTag *t;
	GtWindow *cw;
	GtCursor *cr;
	Bool oev=True;
	ulong em;
	int pm=GrabModeAsync,km=GrabModeAsync;
	Window conf=None;
	Cursor curs=None;
	
	em=GTWPG_EVENTMASK;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_OwnerEvents:
				if(t->d.val)
					oev=True;
				else
					oev=False;
				break;
			case GT_EventMask:
				em=t->d.val;
				break;
			case GT_PointerMode:
				pm=t->d.val;
				break;
			case GT_KeyboardMode:
				km=t->d.val;
				break;
			case GT_ConfineWindow:
				if((cw=(GtWindow *)t->d.ptr))
					conf=cw->XWin();
				break;
			case GT_Cursor:
				if((cr=(GtCursor *)t->d.ptr))
					curs=cr->XCurs();
				break;
		}
	}
	if(XGrabPointer(dsp->XDsp(),win,oev,em,pm,km,conf,curs,tm)==GrabSuccess)
		return(TRUE);
	else
		return(FALSE);
}

/************************************************************************\
*
* Grab pointer in window.
*
\************************************************************************/
sbool GtWindow::GrabPointerV(Time tm,ulong tag,...)
{
	va_list va;
	sbool ret;
	int p;

	va_start(va,tag);
	p=GtBeginVaTags();
	ret=this->GrabPointer(tm,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(ret);
}

/************************************************************************\
*
* Ungrab pointer from window.
*
\************************************************************************/
void GtWindow::UnGrabPointer(Time tm)
{
	XUngrabPointer(dsp->XDsp(),tm);
}
